import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.print.attribute.standard.JobMessageFromOperator;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JComponent;

/**
 * Grafická třída ke hře s odebíráním sirek.
 * 
 * @author     Luboš Pavlíček, Jarmila Pavlíčková
 * @version    2.0 (18. října 2010)
 */
public class GrafikaSirek {
    private JFrame okno; // grafické okno
    private JTextField hromadkaPocetTF; // TF s počtem sirek na začátku,
    private JTextField hromadkaZbyvaTF; // TF s počtem zbývajících sirek  
    private JTextField hracOdebiraTF; // TF, kde hráč zadá, kolik odebírá sirek
    private JTextField pocitacOdebiraTF; // TF s počtem sirek, které odebral počítač
    private JLabel upozorneniLabel; // popiska s upozorněním
    private JPanel upozorneniPanel; // panel s upozorněním

    private IPrubehHry hra; 
    private IPravidla pravidla;

    private static Color colorPozadi = new Color(141, 195, 216); // 0x3aa6d0
    private static Color colorNadpis = new Color(0x024c68);
    private static Color colorUpozorneni = new Color(255, 215, 115);
    private static Color colorWarning = new Color(255, 139, 115);
    private static Color colorTextPravidla = new Color(166, 118, 0);
    private static Color colorTextVstup = new Color(255, 44, 0);
    private static Color colorTextVstupDis = new Color(166, 118, 0);

    /**
     * konstruktor grafické třídy
     */
    public GrafikaSirek(IPrubehHry hra) {
        this.hra = hra;
        this.pravidla = hra.vratPravidla();
        okno = new JFrame();
        okno.setTitle("Odebírání sirek");
        okno.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        JPanel oknoPanel = new JPanel();

        hromadkaPocetTF = new JTextField(5);
        hromadkaZbyvaTF = new JTextField(5);
        hracOdebiraTF = new JTextField(5);
        pocitacOdebiraTF = new JTextField(5);

        upozorneniLabel = new JLabel(" ");

        hromadkaPocetTF.setMaximumSize(hromadkaPocetTF.getPreferredSize());
        hromadkaZbyvaTF.setMaximumSize(hromadkaPocetTF.getPreferredSize());
        hracOdebiraTF.setMaximumSize(hromadkaPocetTF.getPreferredSize());
        pocitacOdebiraTF.setMaximumSize(hromadkaPocetTF.getPreferredSize());

        hromadkaPocetTF.setEnabled(true);
        hromadkaZbyvaTF.setEnabled(false);
        hracOdebiraTF.setEnabled(false);
        pocitacOdebiraTF.setEnabled(false);

        hromadkaPocetTF.setForeground(colorTextVstup);
        hromadkaZbyvaTF.setForeground(colorTextVstup);
        hracOdebiraTF.setForeground(colorTextVstup);
        pocitacOdebiraTF.setForeground(colorTextVstup);

        hromadkaPocetTF.setHorizontalAlignment(JTextField.CENTER);
        hromadkaZbyvaTF.setHorizontalAlignment(JTextField.CENTER);
        hracOdebiraTF.setHorizontalAlignment(JTextField.CENTER);
        pocitacOdebiraTF.setHorizontalAlignment(JTextField.CENTER);

        hromadkaPocetTF.setDisabledTextColor(colorTextVstupDis);
        hromadkaZbyvaTF.setDisabledTextColor(colorTextVstupDis);
        hracOdebiraTF.setDisabledTextColor(colorTextVstupDis);
        pocitacOdebiraTF.setDisabledTextColor(colorTextVstupDis);

        hromadkaPocetTF.addActionListener(new HromadkaPocetAL());
        hracOdebiraTF.addActionListener(new HracOdebiraAL());

        okno.setLayout(new BoxLayout(okno.getContentPane(),
                BoxLayout.PAGE_AXIS));

        JLabel nadpis = new JLabel("Odebírání sirek");
        nadpis.setForeground(colorNadpis);
        nadpis.setFont(new Font("Serif", Font.BOLD, 20));
        JPanel nadpisPanel = new JPanel();
        nadpisPanel.add(nadpis);
        nadpisPanel.setBackground(colorPozadi);
        okno.add(nadpisPanel);
        okno.add(Box.createVerticalStrut(3));

        JPanel hromadkaPocetPanel = new JPanel();
        hromadkaPocetPanel.setLayout(new BoxLayout(hromadkaPocetPanel,
                BoxLayout.LINE_AXIS));
        hromadkaPocetPanel.add(new JLabel("počet sirek na začátku: "));
        hromadkaPocetPanel.add(Box.createHorizontalGlue());
        hromadkaPocetPanel.add(hromadkaPocetTF);
        hromadkaPocetPanel.setBackground(colorPozadi);
        okno.add(hromadkaPocetPanel);
        okno.add(Box.createVerticalStrut(3));

        JPanel hromadkaZbyvaPanel = new JPanel();
        hromadkaZbyvaPanel.setLayout(new BoxLayout(hromadkaZbyvaPanel,
                BoxLayout.LINE_AXIS));
        hromadkaZbyvaPanel.add(new JLabel("počet sirek nyní: "));
        hromadkaZbyvaPanel.add(Box.createHorizontalGlue());
        hromadkaZbyvaPanel.add(hromadkaZbyvaTF);
        hromadkaZbyvaPanel.setBackground(colorPozadi);
        //		hromadkaZbyvaTF.setVisible(false);
        okno.add(hromadkaZbyvaPanel);
        okno.add(Box.createVerticalStrut(6));

        JPanel hracOdebiraPanel = new JPanel();
        hracOdebiraPanel.setLayout(new BoxLayout(hracOdebiraPanel,
                BoxLayout.LINE_AXIS));
        hracOdebiraPanel.add(new JLabel("hráč odebírá: "));
        hracOdebiraPanel.add(Box.createHorizontalGlue());
        hracOdebiraPanel.add(hracOdebiraTF);
        hracOdebiraPanel.setBackground(colorPozadi);
        //		hracOdebiraTF.setVisible(false);
        okno.add(hracOdebiraPanel);
        okno.add(Box.createVerticalStrut(3));

        JPanel pocitacOdebiraPanel = new JPanel();
        pocitacOdebiraPanel.setLayout(new BoxLayout(pocitacOdebiraPanel,
                BoxLayout.LINE_AXIS));
        pocitacOdebiraPanel.add(new JLabel("počítač odebral: "));
        pocitacOdebiraPanel.add(Box.createHorizontalGlue());
        pocitacOdebiraPanel.add(pocitacOdebiraTF);
        pocitacOdebiraPanel.setBackground(colorPozadi);
        //		pocitacOdebiraTF.setVisible(false);
        okno.add(pocitacOdebiraPanel);
        okno.add(Box.createVerticalStrut(5));

        upozorneniPanel = new JPanel();
        upozorneniPanel.add(upozorneniLabel);
        okno.add(upozorneniPanel);
        upozorneniLabel.setText("zadejte počet sirek na hromádce");
        upozorneniPanel.setBackground(colorUpozorneni);
        okno.add(Box.createVerticalStrut(5));

        JPanel pravidlaPanel = new JPanel();
        pravidlaPanel.add(new JLabel(pravidla.pravidla()));
        pravidlaPanel.setBorder(BorderFactory.createTitledBorder("pravidla"));
        pravidlaPanel.setBackground(colorUpozorneni);
        pravidlaPanel.setForeground(colorTextPravidla);
        okno.add(pravidlaPanel);
        okno.add(Box.createVerticalStrut(20));

        hromadkaPocetTF.requestFocusInWindow();
        ((JComponent) okno.getContentPane()).setBorder(BorderFactory
            .createEmptyBorder(3, 3, 3, 3));
        okno.getContentPane().setBackground(colorPozadi);

        okno.pack();

    }

    /**
     * metoda zobrazi graficke okno kalkulacky na obrazovce
     */
    public void setVisible(boolean visible) {
        okno.setVisible(visible);
    }

    /**
     * Metoda, která se volá, když je na tahu počítač.
     */
    private void odebiraPocitac() {
        int pocitac = hra.pocitacOdebira();
        pocitacOdebiraTF.setText(pocitac + "");
        hromadkaZbyvaTF.setText("" + hra.sirekNaHromadce());
        upozorneniLabel.setBackground(colorUpozorneni);
        upozorneniLabel.setText("<html><center>počítač odebral " + pocitac
            + " " + sirkyPad(pocitac)
            + "<br>zadejte počet odebíraných sirek</center></html>");
        hracOdebiraTF.setText("");
        konecHry("počítač");
    }

    private String sirkyPad(int pocet) {
        if (pocet == 1) {
            return "sirku";
        }
        else {
            if (pocet < 5) {
                return "sirky";
            }
            else {
                return "sirek";
            }
        }
    }

    /**
     * metoda, která se volá pro otestování a ošetření konce hry.
     */
    private void konecHry(String kdo) {
        if (hra.jeKonec()) {
            if (pravidla.posledniVyhrava()) {
                JOptionPane.showMessageDialog(null, kdo
                    + " vyhrál, neboť sebral poslední sirku");
            }
            else {
                JOptionPane.showMessageDialog(null, kdo
                    + " prohrál, neboť sebral poslední sirku");
            }
            upozorneniLabel.setText("zadejte počet odebíraných sirek");
            upozorneniPanel.setBackground(colorUpozorneni);
            hromadkaPocetTF.setEnabled(true);
            hromadkaZbyvaTF.setText("");
            hracOdebiraTF.setEnabled(false);
            pocitacOdebiraTF.setText("");
            hromadkaPocetTF.requestFocusInWindow();
        }
    }

    /**
     * Ovladač pro vstupní pole hromadkaPocet.
     */
    private class HromadkaPocetAL implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            String vstup = hromadkaPocetTF.getText();
            int pocet = 0;
            try {
                pocet = Integer.parseInt(vstup);
                if (hra.setPocetSirek(pocet)) {
                    upozorneniPanel.setBackground(colorUpozorneni);
                    hromadkaPocetTF.setEnabled(false);
                    hromadkaZbyvaTF.setVisible(true);
                    hracOdebiraTF.setVisible(true);
                    hracOdebiraTF.setEnabled(true);
                    pocitacOdebiraTF.setVisible(true);
                    hracOdebiraTF.requestFocusInWindow();
                    if (pravidla.zacinaHrac()) {
                        pocitacOdebiraTF.setText("");
                        hromadkaZbyvaTF.setText("" + hra.sirekNaHromadce());
                        upozorneniLabel.setText("zadejte počet odebíraných sirek");
                    }
                    else {
                        odebiraPocitac();
                    }
                }
                else {
                    upozorneniLabel.setText("chybný počet sirek");
                    upozorneniPanel.setBackground(colorWarning);
                }
            }
            catch (NumberFormatException ex) {
                upozorneniLabel.setText("nebylo zadano číslo !!!");
                upozorneniPanel.setBackground(colorWarning);
            }
        }

    }

    /**
     * Ovladač pro vstupní pole hracOdebiraTF.
     */
    private class HracOdebiraAL implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            String vstup = hracOdebiraTF.getText();
            int pocet = 0;
            try {
                pocet = Integer.parseInt(vstup);
                if (hra.hracOdebira(pocet)) {
                    upozorneniPanel.setBackground(colorUpozorneni);
                    hromadkaZbyvaTF.setText("" + hra.sirekNaHromadce());
                    if (hra.jeKonec()) {
                        konecHry("hráč");
                    }
                    else {
                        odebiraPocitac();
                    }
                }
                else {
                    upozorneniLabel.setText("chybný počet sirek");
                    upozorneniPanel.setBackground(colorWarning);
                }
            }
            catch (NumberFormatException ex) {
                upozorneniLabel.setText("nebylo zadano číslo !!!");
                upozorneniPanel.setBackground(colorWarning);
            }
            hracOdebiraTF.setText("");
        }

    }

}
